/**
  ******************************************************************************
  * File Name          : RTC.c
  * Description        : This file provides code for the configuration
  *                      of the RTC instances.
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; Copyright (c) 2019 STMicroelectronics.
  * All rights reserved.</center></h2>
  *
  * This software component is licensed by ST under BSD 3-Clause license,
  * the "License"; You may not use this file except in compliance with the
  * License. You may obtain a copy of the License at:
  *                        opensource.org/licenses/BSD-3-Clause
  *
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "rtc.h"

/* USER CODE BEGIN 0 */
#include <time.h>
uint8_t RTC_isValid=0; /* RTC is Valid */
/* USER CODE END 0 */

RTC_HandleTypeDef hrtc;

/* RTC init function */
void MX_RTC_Init(void)
{

  /** Initialize RTC Only 
  */
  hrtc.Instance = RTC;
  hrtc.Init.HourFormat = RTC_HOURFORMAT_24;
  hrtc.Init.AsynchPrediv = 127;
  hrtc.Init.SynchPrediv = 255;
  hrtc.Init.OutPut = RTC_OUTPUT_DISABLE;
  hrtc.Init.OutPutRemap = RTC_OUTPUT_REMAP_NONE;
  hrtc.Init.OutPutPolarity = RTC_OUTPUT_POLARITY_HIGH;
  hrtc.Init.OutPutType = RTC_OUTPUT_TYPE_OPENDRAIN;
  if (HAL_RTC_Init(&hrtc) != HAL_OK)
  {
    Error_Handler();
  }

}

void HAL_RTC_MspInit(RTC_HandleTypeDef* rtcHandle)
{

  if(rtcHandle->Instance==RTC)
  {
  /* USER CODE BEGIN RTC_MspInit 0 */
	  HAL_RTCEx_EnableBypassShadow(&hrtc);
  /* USER CODE END RTC_MspInit 0 */
    /* RTC clock enable */
    __HAL_RCC_RTC_ENABLE();
  /* USER CODE BEGIN RTC_MspInit 1 */

  /* USER CODE END RTC_MspInit 1 */
  }
}

void HAL_RTC_MspDeInit(RTC_HandleTypeDef* rtcHandle)
{

  if(rtcHandle->Instance==RTC)
  {
  /* USER CODE BEGIN RTC_MspDeInit 0 */

  /* USER CODE END RTC_MspDeInit 0 */
    /* Peripheral clock disable */
    __HAL_RCC_RTC_DISABLE();
  /* USER CODE BEGIN RTC_MspDeInit 1 */

  /* USER CODE END RTC_MspDeInit 1 */
  }
} 

/* USER CODE BEGIN 1 */
/* RTC start function */
void MX_RTC_Start(void)
{
  RTC_TimeTypeDef sTime;
  RTC_DateTypeDef sDate;
  RTC_AlarmTypeDef sAlarm;
  uint8_t i;

    /**Initialize RTC and set the Time and Date
    */
	CmdLine_RegisterCmd("rtcs", "dddddd", RTC_DBG, 0);
	CmdLine_RegisterCmd("rtcg", "", RTC_DBG, 1);
/*	__HAL_RCC_PWR_CLK_ENABLE();
	HAL_PWR_EnableBkUpAccess();
*/
	if (HAL_RTCEx_BKUPRead(&hrtc, RTC_MAGIC_NUMBER_REG_NUMBER) == RTC_MAGIC_NUMBER)
	{
		RTC_isValid = 1;
		return;
	}
	else RTC_isValid = 0;	//Set Flag to not valid RTC Clock (Power is powered OFF.)
}

void RTC_SetDate(RTC_DateTypeDef sDate)
{
	if (HAL_RTC_SetDate(&hrtc, &sDate, RTC_FORMAT_BIN) != HAL_OK)
	{
		Error_Handler();
	}
	else
	{
		HAL_RTCEx_BKUPWrite(&hrtc, RTC_MAGIC_NUMBER_REG_NUMBER, RTC_MAGIC_NUMBER);
		RTC_isValid = 1;
	}
}

void RTC_SetTime(RTC_TimeTypeDef sTime)
{
	if (HAL_RTC_SetTime(&hrtc, &sTime, RTC_FORMAT_BIN) != HAL_OK)
	{
		Error_Handler();
	}
}

time_t RTC_GetTimestamp(void)
{
	RTC_TimeTypeDef sTime;
	RTC_DateTypeDef sDate;
	struct tm currTime;

	HAL_RTC_GetTime(&hrtc, &sTime, RTC_FORMAT_BIN);
	HAL_RTC_GetDate(&hrtc, &sDate, RTC_FORMAT_BIN);
	currTime.tm_year = sDate.Year + RTC_ZERO_YEAR;  /* Years since 1900 : 0-99 range */
	currTime.tm_mon = sDate.Month - 1; /* Months *since* january 0-11 : RTC_Month_Date_Definitions 1 - 12 */
	currTime.tm_mday = sDate.Date;
	currTime.tm_hour = sTime.Hours;
	currTime.tm_min = sTime.Minutes;
	currTime.tm_sec = sTime.Seconds;
	return mktime(&currTime);
}

#if defined(USE_CMDLINE)
unsigned char RTC_DBG(tPar *pPar, unsigned char ucParNum, unsigned char ucSwitch)
{
	RTC_TimeTypeDef sTime;
	RTC_DateTypeDef sDate;

	switch(ucSwitch)
	{
	case 0 :	//rtcs set time and date
		  sTime.Hours = pPar[3].d;
		  sTime.Minutes = pPar[4].d;
		  sTime.Seconds = pPar[5].d;
		  sTime.DayLightSaving = RTC_DAYLIGHTSAVING_NONE;
		  sTime.StoreOperation = RTC_STOREOPERATION_RESET;
		  sDate.WeekDay = RTC_WEEKDAY_MONDAY;
		  sDate.Month = (uint8_t)pPar[1].d;
		  sDate.Date = pPar[2].d;
		  sDate.Year = pPar[0].d - RTC_ZERO_YEAR;
		  RTC_SetDate(sDate);
		  RTC_SetTime(sTime);
		break;
	case 1 :	//rtcg get time and date
		  HAL_RTC_GetTime(&hrtc, &sTime, RTC_FORMAT_BIN);
		  HAL_RTC_GetDate(&hrtc, &sDate, RTC_FORMAT_BIN);
		  printf("RTC date(yyyy-mm-dd)= %d-%d-%d time(hh-mm-ss)=%02d:%02d:%02d\n",  sDate.Year+RTC_ZERO_YEAR, sDate.Month,  sDate.Date, sTime.Hours, sTime.Minutes, sTime.Seconds);
		  printf("RTC: %ld\n",  (uint32_t)RTC_GetTimestamp());
		break;

	}
	return 0;
}
#endif
/* USER CODE END 1 */

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
